/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <X11/Xatom.h>
#include <Xm/XmAll.h>

static GC gc;

static void InputEventCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    XmDrawingAreaCallbackStruct *cb = (XmDrawingAreaCallbackStruct *)call_data;
    int x, y;

    /* Draw a square where button is pressed */
    if (cb->event->type == ButtonPress &&
        cb->event->xbutton.button == Button1) {
        x = cb->event->xbutton.x;
        y = cb->event->xbutton.y;
        XDrawRectangle(XtDisplay(w), cb->event->xany.window, gc, x, y, 8, 8);
    }
}

static void NonMaskEH(w, client_data, event, dispatch)
    Widget     w;
    XtPointer  client_data;
    XEvent    *event;
    Boolean   *dispatch;
{
    Atom custom, clear;

    /* Atoms used on inter client communication */
    custom = XmInternAtom(XtDisplay(w), "CUSTOM MESSAGE", False);
    clear = XmInternAtom(XtDisplay(w), "CLEAR WINDOW", False);

    /* If the type is custom and the data is clear, clear the window */
    if (event->type == ClientMessage &&
         event->xclient.message_type == custom &&
          event->xclient.data.l[0] == clear)
        XClearWindow(XtDisplay(w), XtWindow(w));
}

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Widget toplevel, bboard, canvas;
    Arg    al[20];
    int    ac;
    Window window;
    Atom   atom;

    XtSetLanguageProc(NULL, NULL, NULL);

    ac = 0;
    toplevel = XtAppInitialize(&app_context, "Recvevent", NULL, 0,
                               &argc, argv, NULL, al, ac);

    ac = 0;
    bboard = XmCreateBulletinBoard(toplevel, "bboard", al, ac);
    XtManageChild(bboard);

    ac = 0;
    XtSetArg(al[ac], XmNwidth, 300); ac++;
    XtSetArg(al[ac], XmNheight, 300); ac++;
    canvas = XmCreateDrawingArea(bboard, "canvas", al, ac);
    XtManageChild(canvas);

    /* Callback function called when user input */
    XtAddCallback(canvas, XmNinputCallback, InputEventCB, NULL);
    /* Event handler called when receiving a client message event */
    XtAddEventHandler(canvas, NoEventMask, True, NonMaskEH, NULL);

    XtRealizeWidget(toplevel);

    gc = XCreateGC(XtDisplay(canvas), XtWindow(canvas), NULL, NULL);
    XSetForeground(XtDisplay(canvas), gc, BlackPixelOfScreen(XtScreen(canvas)));

    /* Register a window ID on the property of root window */
    window = XtWindow(canvas);
    atom = XmInternAtom(XtDisplay(canvas), "CANVAS WINDOW", False);
    XChangeProperty(XtDisplay(canvas), RootWindowOfScreen(XtScreen(canvas)),
                    atom, XA_WINDOW, 32, PropModeReplace,
                    (unsigned char *)&window, 1);

    XtAppMainLoop(app_context);
}
