/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <Xm/XmAll.h>
#include <signal.h>

static XtSignalId signal_id;
static int sig_count;  /* The number of the received signal */

/* (3) Signal handler */
static void sig_handler(sig)
    int sig;
{
    printf("A signal is received.\n");
    /* Record the signal arrival and specify the signal callback to be called later */
    XtNoticeSignal(signal_id);
    sig_count++;  /* count the number of received signal */
}

/* (1) Signal Callback */
static void SignalCB(client_data, id)
    XtPointer   client_data;  /* Client data passed at the registration */
    XtSignalId *id;           /* The ID of this signal callback */
{
    /* Block the signal during the signal processing */
    sigset_t sig_mask, save_mask;
    sigemptyset(&sig_mask);
    sigaddset(&sig_mask, SIGUSR1);
    sigprocmask(SIG_BLOCK, &sig_mask, &save_mask);

    /* Perform actual signal processing here */
    printf("Signal Callback:  %d signals were sent.\n", sig_count);

    sigprocmask(SIG_SETMASK, &save_mask, NULL); /* Restore from block */
    sig_count = 0;  /* reset the number of the signal */
}

/* Normal callback function */
static void ButtonCB(w, client_data, call_data)
    Widget     w;
    XtPointer  client_data;
    XtPointer  call_data;
{
    int i;

    printf("Button Callback starts.\n");
    for (i = 0; i < 10; i++) {
        printf("Button Callback is processed.\n");
        sleep(1);  /* perform some work for button press */
    }
    printf("Button Callback ends.\n");
}

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Widget toplevel, panel, button;
    Arg    al[20];
    int    ac;
    struct sigaction sigact;

    XtSetLanguageProc(NULL, NULL, NULL);

    ac = 0;  /* X11R6 initialization function */
    toplevel = XtOpenApplication(&app_context, "Signalcb", NULL, 0,
                        &argc, argv, NULL, sessionShellWidgetClass, al, ac);

    /* (4) Register signal handler */
    sigact.sa_handler = sig_handler;
    sigemptyset(&sigact.sa_mask);
    sigaddset(&sigact.sa_mask, SIGUSR2);
    sigact.sa_flags = NULL;
    sigaction(SIGUSR1, &sigact, NULL);
    sig_count = 0;

    /* (2) Register signal callback */
    signal_id = XtAppAddSignal(app_context, SignalCB, NULL);

    ac = 0;
    panel = XmCreateRowColumn(toplevel, "panel", al, ac);
    XtManageChild(panel);

    ac = 0;
    button = XmCreatePushButton(panel, "button", al, ac);
    XtManageChild(button);
    XtAddCallback(button, XmNactivateCallback, ButtonCB, NULL);

    XtRealizeWidget(toplevel);
    XtAppMainLoop(app_context);
}
